#!/usr/bin/env node
/**
 * @ClassName xsyRequest
 * @Description 统一拦截请求
 * @Author wangtx@xiaoshouyi.com
 * @Date 2019/4/11 18:55
 **/
const logger = require('./xsyLog').logger('xsyRequest');
const request = require('request');
const SqliteDB = require('./xsySql').SqliteDB;
const colors = require('colors');

var sqliteDB = new SqliteDB();

function xsyRequest(options, callback) {
    preInterceptor(options, function (options) {
        request(options,function (error, response, body) {
            afterInterceptor(error, response, body, function (error, response, body) {
                callback(error, response, body);
            });
        });
    });
}

/**
 * 前置拦截器，拼接域名以及在header中加上token
 * @param options
 * @param callback
 */
function preInterceptor(options, callback) {
    // options = JSON.parse(JSON.stringify(options));
    let optionUrl = options["url"].replace(/\s+/g, '').replace(new RegExp('^/+|/+$', 'g'), '');
    let optionHeaders = options["headers"] || {};

    var querySql = 'select * from environment where name = "current"';
    sqliteDB.queryData(querySql, function (objects) {
        let current = JSON.parse(objects[0]);
        let currentUri = current["uri"];
        let currentToken = current["token"];
        if (!currentToken) {
            console.log("Not Logged in!".red);
            logger.error("Current Token is null:"+JSON.stringify(current));
            process.exit(1);
        }
        currentUri = currentUri.replace(new RegExp('^/+|/+$', 'g'), '');
        optionUrl = currentUri + "/" + optionUrl;
        optionHeaders["Authorization"] = currentToken;
        options["url"] = optionUrl;
        options["headers"] = optionHeaders;
        logger.info(options);
        callback(options);
    });
}

function afterInterceptor(error, response, body, callback) {
    if (error) {
        console.log((error.message).red);
        logger.error("xsyRequest after error:" + error.stack);
        process.exit(1);
    }
    if (body) {
        body = JSON.parse(body);
        if ("200" == body["code"]) {
            callback(error, response, body);
        } else if ("1020008" == body["code"]) {
            // 重新登陆不在这里进行，这里尽量业务关联少
            console.log("Your logon has expired. Please log on again to continue!".red);
            logger.error(body);
            process.exit(1);
        } else {
            // console.log(JSON.stringify(body).red);
            logger.error(body);
            // process.exit(1);
            callback(error, response, body);
        }
    }
}

module.exports = xsyRequest;
