#!/usr/bin/env node
/**
 * @ClassName xsySql
 * @Description sqlite3的基础操作
 * @Author wangtx@xiaoshouyi.com
 * @Date 2019/4/9 17:41
 **/

const logger = require('./xsyLog').logger('xsySql');
const fs = require('fs');
const async = require('async');
const sqlite3 = require('sqlite3').verbose();
const path = require('path');
const dbfile = path.join(path.resolve(__dirname), '../lib/xsy.db');

var DB = DB || {};
var createEnvironmentTableSql = "create table if not exists environment(name varchar(32) primary key,e_name varchar(32),uri varchar(32),username varchar(32),password varchar(32),client_id varchar(100),client_secret varchar(100),client_code varchar(32),token varchar(100));";
var initializeEnvironmentTableSql = "insert into environment(name, e_name, uri, username, password, client_id, client_secret, client_code, token) values(?, ?, ?, ?, ?, ?, ?, ?, ?);";

var initializeEnvironment = function (callback) {
    try {
        logger.info("initialize environment table!");
        DB.db.serialize(function () {
            DB.db.run(createEnvironmentTableSql, function (err) {
                if (null != err) {
                    logger.error("Create table error:" + err);
                    return;
                } else {
                    logger.info("Create tabke success");
                }
            });
            let st = DB.db.prepare(initializeEnvironmentTableSql);
            let currentData = ["current", "", "", "", "", "", "", "", ""];
            st.run(currentData);
            st.finalize();
            callback();
        });
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

{
    try {
        let exist = fs.existsSync(dbfile);
        DB.db = new sqlite3.Database(dbfile);
        let state = fs.statSync(dbfile);
        if (!exist || state.size == 0) {
            logger.info("Creating db file!");
            try {
                fs.openSync(file, 'w');
            } catch (e) {
                logger.info("First create file!");
            }
            async.series([
                function (callback) {
                    initializeEnvironment(function () {
                        callback;
                    });
                }
            ]);
        }
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
}

DB.SqliteDB = function () {};

DB.SqliteDB.prototype.createTable = function (sql) {
    try {
        logger.index("Create table sql:" + sql);
        DB.db.serialize(function () {
            DB.db.run(sql, function (err) {
                if (null != err) {
                    logger.error("Create table error:" + err);
                    return;
                }
            })
        })
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

/**
 *
 * @param sql
 * @param objects
 */
DB.SqliteDB.prototype.insertData = function (sql, objects) {
    try {
        DB.db.serialize(function () {
            logger.info("insert sql:" + sql);
            logger.info("insert value:" + JSON.stringify(objects));
            let st = DB.db.prepare(sql);
            for (var i = 0; i < objects.length; ++i) {
                st.run(objects[i]);
            }
            st.finalize();
        });
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

/**
 *
 * @param sql
 * @param objects
 */
DB.SqliteDB.prototype.insertDataAsync = function (sql, objects, callback) {
    try {
        DB.db.serialize(function () {
            logger.info("insert sql:" + sql);
            logger.info("insert value:" + JSON.stringify(objects));
            let st = DB.db.prepare(sql);
            for (var i = 0; i < objects.length; ++i) {
                st.run(objects[i]);
            }
            st.finalize();
            callback();
        });
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

DB.SqliteDB.prototype.queryData = function (sql, callback) {
    try {
        logger.info("query sql:" + sql);
        DB.db.all(sql, function (err, rows) {
            if (null != err) {
                logger.error("query error:" + err);
                return;
            }
            if (callback) {
                // 统一把返回值转下JSON
                let res = [];
                rows.forEach(function (value, index) {
                    res[index] = JSON.stringify(value);
                });
                callback(res);
            }
        });
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

DB.SqliteDB.prototype.executeSql = function (sql) {
    try {
        logger.info("execute sql:" + sql);
        DB.db.run(sql, function (err) {
            if (null != err) {
                logger.error("execute sql error:" + err);
                return;
            }
        })
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

DB.SqliteDB.prototype.executeSqlAsync = function (sql, callback) {
    try {
        logger.info("execute sql:" + sql);
        DB.db.run(sql, function (err) {
            if (null != err) {
                logger.error("execute sql error:" + err);
            }
            callback();
        })
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

DB.SqliteDB.prototype.close = function () {
    try {
        DB.db.close();
    } catch (e) {
        console.log((e.message).red);
        logger.error(e.stack);
        process.exit(1);
    }
};

exports.SqliteDB = DB.SqliteDB;
