#!/usr/bin/env node
/**
 * @ClassName init
 * @Description TODO
 * @Author wangtx@xiaoshouyi.com
 * @Date 2019/4/10 18:38
 **/
const logger = require('../../core/xsyLog').logger('module_project_init');
const xsyRequest = require('../../core/xsyRequest');
const loginUtils = require('../../utils/loginUtils');
const environmentUtils = require('../../utils/environmentUtils');
const fs = require("fs");
const path = require("path");
const inquirer = require('inquirer');
const colors = require('colors');
const unzip = require("unzip");
const request = require('request');

const ingageSdkfile = path.join(path.resolve(__dirname), '../../lib/ingageSDK.jar');
let projectName = "";
let packageName = "";
let descriptionStr = "";
let api_status = true;
let trigger_status = true;

function main() {
    const promptList = [{
        type: 'input',
        message: 'Project name:',
        name: 'projectName',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            if (val === "") {
                return "Project name can not be empty!";
            } else if (val.length > 50) {
                return "Project name is too long!";
            } else {
                return true;
            }
        }
    }, {
        type: 'input',
        message: 'Package name:',
        name: 'packageName',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            val = val.replace(new RegExp('^\\.+|\\.+$', 'g'), '');
            if (val === "") {
                return "Package name can not be empty!";
            } else if (val.length > 50) {
                return "Package name is too long!";
            } else if (!val.startsWith('other')) {
                return "the path must be start with other!";
            } else if (val.split('.').length != 3) {
                return "the path must be three levels!";
            } else {
                return true;
            }
        }
    }, {
        type: 'input',
        message: 'Project description:',
        name: 'descriptionStr',
        validate: function (val) {
            val = val.trim();
            if (val.length > 50) {
                return "Project description is too long!";
            } else {
                return true;
            }
        }
    }, {
        type: 'confirm',
        message: 'create apiUrl.xml?',
        name: 'api_status',
        default: true
    }, {
        type: 'confirm',
        message: 'create scriptTrigger.xml?',
        name: 'trigger_status',
        default: true
    }];

    inquirer.prompt(promptList).then(answers => {
        projectName = answers.projectName;
        packageName = answers.packageName;
        descriptionStr = answers.descriptionStr;
        api_status = answers.api_status;
        trigger_status = answers.trigger_status;
        init_folder(function () {
            console.log("Initialization Successful!".green);
        });
    })
}

function init_folder(callback) {
    pull_from_remote(function (ret_flg) {
        if (ret_flg) {
            var currentDir = process.cwd();
            currentDir = path.join(currentDir, projectName);
            // 创建projectName
            if (!fs.existsSync(currentDir)) {
                fs.mkdirSync(currentDir);
            }
            write_xsy();

            // 创建src目录
            srcPath = path.join(currentDir, "src");
            if (!fs.existsSync(srcPath)) {
                fs.mkdirSync(srcPath);
                if (trigger_status) {
                    write_trigger_template(srcPath);
                }
                if (api_status) {
                    write_api_template(srcPath);
                }
            }

            libPath = path.join(currentDir, "lib");
            if (!fs.existsSync(libPath)) {
                fs.mkdirSync(libPath);
                copy_sdk(libPath);
                downloadModel(libPath);
            }

            fs.createReadStream(path.join(process.cwd(), packageName + ".zip")).pipe(unzip.Extract({path: path.join(process.cwd(), projectName, "src")}));
            fs.unlink(path.join(process.cwd(), packageName + ".zip"), function (error) {});
            callback();
        } else {
            mkdirs(function () {
                callback();
            })
        }
    });
}

function pull_from_remote(callback) {
    xsyRequest({
        url: '/rest/metadata/v2.0/scripts/packages?packageName=' + packageName,
        method: 'GET'
    }, function (error, response, body) {
        if (body.code === "200") {
            if (body.data.records.uploadFile == null) {
                callback(false)
            }  else {
                download_remote_package(body, function () {
                    callback(true)
                });
            }
        } else if (body.code === "290026") {
            callback(false);
        } else if (body.code === "290023") {
            callback(false);
        } else {
            console.log(JSON.stringify(body).red);
            process.exit(1);
        }
    });
}

function download_remote_package(jsonStr, callback) {
    let modelStream = fs.createWriteStream(path.join(process.cwd(), packageName + ".zip"));
    request(jsonStr.data.records.uploadFile).pipe(modelStream).on('close', callback);
}


function mkdirs(callback) {
    let currentDir = process.cwd();
    currentDir = path.join(currentDir, projectName);
    // 创建projectName
    if (!fs.existsSync(currentDir)) {
        fs.mkdirSync(currentDir);
    }

    write_xsy();

    // 创建src目录
    srcPath = path.join(currentDir, "src");
    if (!fs.existsSync(srcPath)) {
        fs.mkdirSync(srcPath);
        if (trigger_status) {
            write_trigger_template(srcPath);
        }
        if (api_status) {
            write_api_template(srcPath);
        }
    }

    libPath = path.join(currentDir, "lib");
    if (!fs.existsSync(libPath)) {
        fs.mkdirSync(libPath);
        copy_sdk(libPath);
        downloadModel(libPath);
    }

    for (const dirname of packageName.split('.')) {
        srcPath = path.join(srcPath, dirname);
        if (!fs.existsSync(dirname)) {
            fs.mkdirSync(srcPath);
        }
    }
    callback();
}

/**
 * 给指定问价写入配置信息
 * @param pPath
 */
function write_xsy() {
    currentDir = process.cwd();
    pPath = path.join(currentDir, projectName, ".xsy");
    let params = {
        "packageName": packageName,
        "projectName": projectName,
        "projectDescription": descriptionStr
    };
    fs.writeFileSync(pPath, JSON.stringify(params));
}


/**
 * 在指定path创建一个api的XML模板
 * @param fileDir
 */
function write_api_template(fileDir) {
    filePath = path.join(fileDir, 'apiUrl.xml');
    fileContent = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
        "<configs>\n" +
        "\t<config>\n" +
        "\t\t<api>\n" +
        "\t\t\t<class>" + packageName + ".ApiDemoImpl</class>\n" +
        "\t\t\t<url>/query</url>\n" +
        "\t\t</api>\n" +
        "\t</config>\n" +
        "</configs>\n";
    fs.writeFileSync(filePath, fileContent);
}

/**
 * 在指定path创建一个trigger的XML模板
 * @param fileDir
 */
function write_trigger_template(fileDir) {
    filePath = path.join(fileDir, 'scriptTrigger.xml');
    fileContent = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
        "<configs>\n" +
        "\t<config>\n" +
        "\t\t<trigger>\n" +
        "\t\t\t<!-- From :  https://api.xiaoshouyi.com/data/v1/picks/dimension/belongs-->\n" +
        "\t\t\t<object>account</object>\n" +
        "\t\t\t<!-- Options: add / update / delete-->\n" +
        "\t\t\t<operate>add</operate>\n" +
        "\t\t\t<!-- Options: before / after-->\n" +
        "\t\t\t<position>after</position>\n" +
        "\t\t\t<order>1</order>\n" +
        "\t\t\t<!-- Caution: match case / spell / full path-->\n" +
        "\t\t\t<class>" + packageName + ".TriggerDemoImpl</class>\n" +
        "\t\t</trigger>\n" +
        "\t\t<wfmTrigger>\n" +
        "\t\t\t<object>account</object>\n" +
        "\t\t\t" + packageName + ".WfmTriggerDemoImpl</class>\n" +
        "\t\t</wfmTrigger>\n" +
        "\t\t<schedule>\n" +
        "\t\t\t<class>" + packageName + ".ScheduleDemoImpl</class>\n" +
        "\t\t</schedule>\n" +
        "\t\t<DSUserTrigger>\n" +
        "\t\t\t<object>account</object>\n" +
        "\t\t\t<apikey>youCanNamingApiKeyLikeThis</apikey>\n" +
        "\t\t\t<class>" + packageName + ".DsUserTriggerDemoImpl</class>\n" +
        "\t\t</DSUserTrigger>\n" +
        "\t</config>\n" +
        "</configs>\n";
    fs.writeFileSync(filePath, fileContent);
}

/**
 * copy sdk 到指定目录
 * @param fileDir
 */
function copy_sdk(fileDir) {
    // if (sdk_version !== undefined) {
        filePath = path.join(fileDir, 'ingageSDK.jar');
        fs.writeFileSync(filePath, fs.readFileSync(ingageSdkfile));
    // }
}

/**
 * 下载modelJar到当前工程lib目录下
 */
function downloadModel(projectPath) {
    xsyRequest({
        url: '/rest/metadata/v2.0/scripts/packages/model/jar/url',
        method: 'GET'
    }, function (error, response, body) {
        if (body.code === "200") {
            let modelStream = fs.createWriteStream(path.join(projectPath, "model.jar"));
            request(body.data.records).pipe(modelStream).on('close', function () {
            });
        } else {
            console.log(JSON.stringify(body).red);
            process.exit(1);
        }
    });
}

exports.main = main;
