#!/usr/bin/env node
/**
 * @ClassName push
 * @Description TODO
 * @Author wangtx@xiaoshouyi.com
 * @Date 2019/4/10 18:39
 **/
const logger = require('../../core/xsyLog').logger('module_script_push');
const xsyRequest = require('../../core/xsyRequest');
const loginUtils = require('../../utils/loginUtils');
const commonUtils = require('../../utils/commonUtils');
const environmentUtils = require('../../utils/environmentUtils');
const fs = require("fs");
const path = require("path");
const inquirer = require('inquirer');
const colors = require('colors');
const compressing = require('compressing');
const request = require('request');

let packageName = "";
let projectPath = "";
let token = "";

/**
 * 安装代码包
 */
function install_package() {
    return new Promise((resolve, reject) => {
        let body = {
            packageName: packageName
        };
        xsyRequest({
            url: '/rest/metadata/v2.0/scripts/packages/installations',
            method: 'POST',
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify(body)
        }, function (error, response, body) {
            if (body.code === "200") {
                console.log("Install Success!".green)
                resolve("done");
            } else {
                console.log(JSON.stringify(body).red);
                process.exit(1);
            }
        });
    })
}

/**
 * 上传代码包
 * 必须要有package，不然爆出来String index out of range: -1
 */
function upload_package() {
    return new Promise((resolve, reject) => {
        zipPath = path.join(projectPath, 'upload_source.zip');
        parPath = path.join(projectPath, "src/.");
        // 加上.代表当前目录，不然他只会打包上级目录
        compressing.zip.compressDir(parPath + path.sep + ".", zipPath)
            .then(() => {
                xsyRequest({
                    url: '/rest/metadata/v2.0/scripts/packages/files?packageName=' + packageName,
                    method: 'POST',
                    headers: {'Content-Type': 'application/x-www-form-urlencoded'},
                    formData: {file: fs.createReadStream(zipPath)}
                }, function (error, response, body) {
                    if (body.code === "200") {
                        console.log("Push Success!".green);
                        resolve("done");
                    } else if (body.code === "290025") {
                        console.log("java file don't have package".red);
                        process.exit(1);
                    } else {
                        console.log(JSON.stringify(body).red);
                        process.exit(1);
                    }
                });
            });
    })
}


/**
 * 先卸载代码包
 */
function uninstall_package() {
    return new Promise((resolve, reject) => {
        xsyRequest({
            url: '/rest/metadata/v2.0/scripts/packages/installations/' + packageName + "?force=false",
            method: 'DELETE'
        }, function (error, response, body) {
            if (body.code === "200") {
                resolve("done");
            } else if (body.code === "290030") {
                xsyRequest({
                    url: '/rest/metadata/v2.0/scripts/packages/installations/' + packageName + "?force=true",
                    method: 'DELETE'
                }, function (error, response, body) {
                    if (body.code === "200") {
                        resolve("done");
                    } else {
                        console.log(JSON.stringify(body).red);
                        process.exit(1);
                    }
                });
            } else {
                console.log(JSON.stringify(body).red);
                process.exit(1);
            }
        });
    })
}

/**
 * 创建代码包
 * @param callback
 */
function create_package() {
    return new Promise((resolve, reject) => {
        commonUtils.get_project_path(process.cwd(), function (pPath) {
            projectPath = pPath;
            commonUtils.get_xsy_json(function (xsyJson) {
                packageName = xsyJson["packageName"];
                xsyRequest({
                    url: '/rest/metadata/v2.0/scripts/packages?packageName=' + packageName,
                    method: 'GET'
                }, function (error, response, body) {
                    if (body.code === "200") {
                        resolve("done");
                    } else if (body.code === "290026" || body.code === "290023") {
                        const promptList = [{
                            type: 'confirm',
                            message: 'This project does not exist on sever, would you like create a new project?',
                            name: 'is_agree',
                            default: true
                        }];

                        inquirer.prompt(promptList).then(answers => {
                            if (answers.is_agree == true) {
                                let body = {
                                    packageName: xsyJson["packageName"],
                                    name: xsyJson["projectName"],
                                    content: xsyJson["projectDescription"]
                                };
                                console.log(body)
                                xsyRequest({
                                    url: '/rest/metadata/v2.0/scripts/packages',
                                    method: 'POST',
                                    headers: {'Content-Type': 'application/json'},
                                    body: JSON.stringify(body)
                                }, function (error, response, body) {
                                    if (body.code === "200") {
                                        console.log("Create Success!".green);
                                        resolve("done");
                                    } else {
                                        console.log(JSON.stringify(body).red);
                                        process.exit(1);
                                    }
                                });
                            } else {
                                process.exit(1);
                            }
                        });
                    } else {
                        console.log(JSON.stringify(body).red);
                        process.exit(1);
                    }
                });
            });
        });
    })
}

async function main() {
    await create_package();
    console.log("Pushing...".gray)
    await uninstall_package();
    await upload_package();
    await install_package();
}

exports.main = main;
