#!/usr/bin/env node

const process = require("process");
const request = require("request");
const path = require("path");
const fs = require("fs");
const xsyRequest = require('../core/xsyRequest');

/**
 * 通过包名获取该包在s3上面的地址
 * @param packageName
 * @param callback
 */
function get_zip_url(packageName, baseUrl, callback) {
    xsyRequest({
        url: '/rest/metadata/v2.0/scripts/packages?packageName=' + packageName,
        method: 'GET'
    }, function (error, response, body) {
        if (body.code === "200") {
            let jsonData = body.data;
            if (!fs.existsSync(baseUrl)) {
                console.log("No such file or directory!".red);
                callback(false);
            } else {
                let zipName = path.join(baseUrl, jsonData.records.codePackageName);
                download_stream(jsonData.records.uploadFile, zipName, function () {
                    callback(true);
                });
            }
        } else {
            console.log(JSON.stringify(body).red);
            process.exit(1);
        }
    });
}

/**
 * 通过url，和zip包全路径下载文件
 * @param urlStr
 * @param zipName
 * @param callback
 */
function download_stream(urlStr, zipName, callback) {
    let zipStream = fs.createWriteStream(zipName + ".zip");
    request(urlStr).pipe(zipStream).on('close', callback);
}

/**
 * 获取工程的根路径，通过判断.xsy文件来决定
 * @param cPath
 * @param callback
 */
function get_project_path(cPath, callback) {
    let parentPath = path.resolve(cPath, '..');
    if (parentPath !== cPath) {
        let xsy_flg = false;
        if (fs.statSync(cPath).isDirectory()) {
            let files = fs.readdirSync(cPath);
            files.forEach(function (item, index) {
                let fPath = path.join(cPath, item);
                let stat = fs.statSync(fPath);
                if (stat.isFile() && item === ".xsy") {
                    xsy_flg = true;
                    callback(cPath);
                }
            });
            if (!xsy_flg) {
                get_project_path(parentPath, function (currentPath) {
                    callback(currentPath);
                })
            }
        }
    } else {
        console.log("can't find .xsy file!".red);
        process.exit(1);
    }
}

function get_xsy_json(callback) {
    let currentPath = process.cwd();
    get_project_path(currentPath, function (projectPath) {
        if (projectPath !== "") {
            let xsyFile = path.join(projectPath, ".xsy");
            let xsyStr = fs.readFileSync(xsyFile, 'utf-8');
            let xsyJson = JSON.parse(xsyStr);
            callback(xsyJson);
        }
    })
}

function get_xsy_value_by_key(key, callback) {
    let currentPath = process.cwd();
    get_project_path(currentPath, function (projectPath) {
        if (projectPath !== "") {
            let xsyFile = path.join(projectPath, ".xsy");
            let xsyStr = fs.readFileSync(xsyFile, 'utf-8');
            let xsyJson = JSON.parse(xsyStr);
            callback(xsyJson[key]);
        }
    })
}

function set_xsy_value_by_key(key, newValue, callback) {
    let currentPath = process.cwd();
    get_project_path(currentPath, function (projectPath) {
        if (projectPath !== "") {
            let xsyFile = path.join(projectPath, ".xsy");
            let xsyStr = fs.readFileSync(xsyFile, 'utf-8');
            let xsyJson = JSON.parse(xsyStr);
            xsyJson[key] = newValue;
            fs.writeFile(xsyFile, JSON.stringify(xsyJson), 'utf8', (err) => {
                if (err) {
                    callback(false);
                } else {
                    callback(true);
                }
            });
        }
    })
}

exports.get_zip_url = get_zip_url;
exports.get_project_path = get_project_path;
exports.get_xsy_value_by_key = get_xsy_value_by_key;
exports.set_xsy_value_by_key = set_xsy_value_by_key;
exports.get_xsy_json = get_xsy_json;
