#!/usr/bin/env node
/**
 * @ClassName environmentUtils
 * @Description TODO
 * @Author wangtx@xiaoshouyi.com
 * @Date 2019/4/10 15:38
 **/
const colors = require('colors');
const Environment = require('./Environment');
const SqliteDB = require('../core/xsySql').SqliteDB;
const logger = require('../core/xsyLog').logger('environmentUtils');

var environmentUtils = environmentUtils || {};
var sqliteDB = new SqliteDB();

/**
 * 获取所有环境名称
 * @param callback
 */
environmentUtils.getAllName = function (callback) {
    let getAllNameSql = "SELECT name FROM 'environment' WHERE name != 'current';";
    sqliteDB.queryData(getAllNameSql, function (objects) {
        let allName = [];
        for (let i in objects) {
            allName.push(JSON.parse(objects[i])["name"]);
        }
        callback(allName);
    });
};

/**
 * 删除一个环境配置，如果当前环境配置也是这个配置，清空，用户自己选择重新登录
 * @param name
 */
environmentUtils.removeEnvironment = function (name) {
    logger.info("begin remove environment:" + name);
    let removeEnvironmentSql = "delete from 'environment' where name = '" + name + "';";
    sqliteDB.executeSql(removeEnvironmentSql);
    let isCurrentSql = "select e_name from 'environment' where name = 'current';";
    sqliteDB.queryData(isCurrentSql, function (objects) {
        let current = JSON.parse(objects[0]);
        if (name === current["e_name"]) {
            let emptyEnvironment = new Environment("", "", "", "", "", "", "", "", "");
            updateCurrent(emptyEnvironment);
        }
    });
};

/**
 * 添加一个环境, 自动切换到该环境
 * @param environment
 * @param callback
 */
environmentUtils.addEnvironment = function (environment, callback) {
    let insertOneSql = "insert into environment(name, e_name, uri, username, password, client_id, client_secret, client_code, token) values(?, ?, ?, ?, ?, ?, ?, ?, ?);"
    let insertValue = [[]];
    insertValue[0].push(environment.name);
    insertValue[0].push(environment.e_name);
    insertValue[0].push(environment.uri);
    insertValue[0].push(environment.username);
    insertValue[0].push(environment.password);
    insertValue[0].push(environment.client_id);
    insertValue[0].push(environment.client_secret);
    insertValue[0].push(environment.client_code);
    insertValue[0].push(environment.token);
    sqliteDB.insertDataAsync(insertOneSql, insertValue, function () {
        updateCurrentAsync(environment, function () {
            callback();
        });
    });
};

/**
 * 切换当前环境变量到指定环境变量
 * @param name
 * @param callback
 */
environmentUtils.changeCurrent = function (name) {
    let environmentSql = "select * from environment where name = '" + name + "'";
    sqliteDB.queryData(environmentSql, function (objects) {
        let environment = JSON.parse(objects[0]);
        updateCurrent(environment);
    })
};

/**
 * 获取当前系统变量token
 * @param callback
 */
environmentUtils.getCurrentToken = function (callback) {
    let getTokenSql = "SELECT token from environment WHERE name = 'current';";
    sqliteDB.queryData(getTokenSql, function (objects) {
        callback(JSON.parse(objects[0])["token"]);
    });
};

/**
 * 获取当前系统变量名称
 * @param callback
 */
environmentUtils.getCurrentName = function (callback) {
    let getCurrentNameSql = "SELECT e_name from environment WHERE name = 'current';";
    sqliteDB.queryData(getCurrentNameSql, function (objects) {
        callback(JSON.parse(objects[0])["e_name"]);
    });
};

environmentUtils.getCurrentHost = function (callback) {
    let getCurrentNameSql = "SELECT uri from environment WHERE name = 'current';";
    sqliteDB.queryData(getCurrentNameSql, function (objects) {
        callback(JSON.parse(objects[0])["uri"]);
    });
};

environmentUtils.getCurrentConfig = function (callback) {
    environmentUtils.getCurrentName(function (name) {
        let getCurrentNameSql = "SELECT * from environment WHERE name = '"+name+"';";
        sqliteDB.queryData(getCurrentNameSql, function (objects) {
            callback(JSON.parse(objects[0]));
        });
    })
};

function updateCurrentAsync(environment, callback) {
    let updateCurrentSql = "UPDATE 'environment' SET 'e_name' = '" + environment.e_name + "', 'uri' = '" + environment.uri + "', " +
        "'username' = '" + environment.username + "', 'password' = '" + environment.password + "', 'client_id' = '" + environment.client_id + "', " +
        "'client_secret' = '" + environment.client_secret + "', 'client_code' = '" + environment.client_code + "', 'token' = '" + environment.token + "' WHERE name = 'current';";
    sqliteDB.executeSqlAsync(updateCurrentSql, function () {
        callback();
    });
}

function updateCurrent(environment) {
    let updateCurrentSql = "UPDATE 'environment' SET 'e_name' = '" + environment.e_name + "', 'uri' = '" + environment.uri + "', " +
        "'username' = '" + environment.username + "', 'password' = '" + environment.password + "', 'client_id' = '" + environment.client_id + "', " +
        "'client_secret' = '" + environment.client_secret + "', 'client_code' = '" + environment.client_code + "', 'token' = '" + environment.token + "' WHERE name = 'current';";
    sqliteDB.executeSql(updateCurrentSql);
}

environmentUtils.updateEnvironment = function (environment) {
    let updateCurrentSql = "UPDATE 'environment' SET 'e_name' = '" + environment.e_name + "', 'uri' = '" + environment.uri + "', " +
        "'username' = '" + environment.username + "', 'password' = '" + environment.password + "', 'client_id' = '" + environment.client_id + "', " +
        "'client_secret' = '" + environment.client_secret + "', 'client_code' = '" + environment.client_code + "', 'token' = '" + environment.token + "' WHERE name = '" + environment.e_name + "';";
    sqliteDB.executeSql(updateCurrentSql);
    updateCurrent(environment);
};

environmentUtils.updateEnvironmentAsync = function (environment, callback) {
    let updateEnvironmentSql = "UPDATE 'environment' SET 'e_name' = '" + environment.e_name + "', 'uri' = '" + environment.uri + "', " +
        "'username' = '" + environment.username + "', 'password' = '" + environment.password + "', 'client_id' = '" + environment.client_id + "', " +
        "'client_secret' = '" + environment.client_secret + "', 'client_code' = '" + environment.client_code + "', 'token' = '" + environment.token + "' WHERE name = '" + environment.e_name + "';";
    sqliteDB.executeSqlAsync(updateEnvironmentSql, function () {
        updateCurrentAsync(environment, function () {
            callback();
        });
    });
};

module.exports = environmentUtils;
