#!/usr/bin/env node
/**
 * @ClassName loginUtils
 * @Description TODO
 * @Author wangtx@xiaoshouyi.com
 * @Date 2019/4/10 15:38
 **/
const request = require("request");
const inquirer = require('inquirer');
const colors = require('colors');
const environmentUtils = require('./environmentUtils');
const logger = require('../core/xsyLog').logger('loginUtils');

var loginUtils = loginUtils || {};

loginUtils.login = function () {
    const promptList = [{
        type: 'input',
        message: 'Please input name:',
        name: 'name',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            if (val === "") {
                return "config name can not be empty!";
            } else if (val.startsWith("{") && val.endsWith("}")) {
                try {
                    let jsonStr = JSON.parse(val);
                    if (jsonStr["name"] === undefined) {
                        return "name undefined";
                    } else if (jsonStr["username"] === undefined) {
                        return "username undefined";
                    } else if (jsonStr["password"] === undefined) {
                        return "password undefined";
                    } else if (jsonStr["client_id"] === undefined) {
                        return "client_id undefined";
                    } else if (jsonStr["client_secret"] === undefined) {
                        return "client_secret undefined";
                    } else if (jsonStr["client_code"] === undefined) {
                        return "client_code undefined";
                    } else if (jsonStr["uri"] === undefined) {
                        return "uri undefined";
                    } else {
                        loginReq(jsonStr, true, function () {
                        })
                    }
                } catch (e) {
                    console.log(e);
                    return "Illegal Characters!";
                }
            } else {
                return true;
            }
        }
    }, {
        type: 'input',
        message: 'Please input username:',
        name: 'username',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            let emailFormat = /^[a-zA-Z0-9_-]+@([a-zA-Z0-9]+\.)+(com|cn|net|org)$/;
            let telephoneFormat = /^[1][3,4,5,7,8][0-9]{9}$/;
            if (val === "") {
                return "username can not be empty!";
            } else if (!emailFormat.test(val) && !telephoneFormat.test(val)) {
                return "Incorrect format: username must be telephone or email format!";
            } else {
                return true;
            }
        }
    }, {
        type: 'password',
        message: 'Please input password:',
        name: 'password',
        mask: '*',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            if (val === "") {
                return "password can not be empty!";
            } else if (val.length > 50) {
                return "password is too long!";
            } else {
                return true;
            }
        }
    }, {
        type: 'input',
        message: 'Please input ClientId:',
        name: 'client_id',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            let strFormat = /^[a-z0-9]+$/;
            if (val === "") {
                return "ClientId can not be empty!";
            } else if (!strFormat.test(val) || val.length != 32) {
                return "ClientId invalid!";
            } else {
                return true;
            }
        }
    }, {
        type: 'input',
        message: 'Please input ClientSecret:',
        name: 'client_secret',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            let strFormat = /^[a-z0-9]+$/;
            if (val === "") {
                return "ClientSecret can not be empty!";
            } else if (!strFormat.test(val) || val.length != 32) {
                return "ClientSecret invalid!";
            } else {
                return true;
            }
        }
    }, {
        type: 'input',
        message: 'Please input SecretCode:',
        name: 'client_code',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            let strFormat = /^[A-Za-z0-9]+$/;
            if (val === "") {
                return "SecretCode can not be empty!";
            } else if (!strFormat.test(val) || val.length != 8) {
                return "SecretCode invalid!";
            } else {
                return true;
            }
        }
    }, {
        type: 'input',
        message: 'Please input Host:',
        name: 'uri',
        validate: function (val) {
            val = val.replace(/\s+/g, '');
            if (val === "") {
                return "uri can not be empty!";
            } else if (!val.startsWith("https://api") || !val.endsWith(".xiaoshouyi.com")) {
                return "uri invalid!";
            } else {
                return true;
            }
        }
    }];

    inquirer.prompt(promptList).then(environment => {
        loginType = "login";
        loginReq(environment, true, function () {

        })
    });
};

/**
 *
 * @param environment
 * @param flg true：新增 false重试
 * @param callback
 */
function loginReq(environment, flg, callback) {
    let requestUrl = environment.uri + "/oauth2/token.action?grant_type=password&client_id=" + environment.client_id +
        "&client_secret=" + environment.client_secret + "&username=" + environment.username + "&password=" + environment.password + environment.client_code;
    logger.info("get token url: " + requestUrl);
    request({
            url: requestUrl,
            method: 'GET'
        }, function (error, response, body) {
            if (body === undefined) {
                console.log(error.message.red);
                process.exit(1);
            } else {
                let jsonStr = JSON.parse(body);
                if (jsonStr["access_token"] !== undefined) {
                    environment.token = jsonStr["access_token"];
                    if (flg) {
                        environment.e_name = environment.name;
                        environmentUtils.addEnvironment(environment, function () {
                            console.log("Login Success!".green);
                            process.exit(0);
                        });
                    } else {
                        environmentUtils.updateEnvironmentAsync(environment, function () {
                            callback();
                        });
                    }
                } else if (jsonStr["error_description"] !== undefined) {
                    console.log((jsonStr["error_description"]).red);
                    process.exit(1);
                } else {
                    console.log((jsonStr).red);
                    process.exit(1);
                }
            }
        }
    );
}

loginUtils.reLogin = function (callback) {
    environmentUtils.getCurrentConfig(function (environment) {
        loginReq(environment, false, function () {
            callback();
        });
    })
};

module.exports = loginUtils;
