#!/usr/bin/env node

const program = require('commander');
const figlet = require('figlet');
const chalk = require('chalk');
const colors = require('colors');
const process = require("process");
const environment = require('./modules/environment');
const download = require('./modules/modeljar/download');
const update = require('./modules/modeljar/update');
const push = require('./modules/script/push');
const pull = require('./modules/script/pull');
const init = require('./modules/project/init');
const loginUtils = require('./utils/loginUtils');
const path = require('path');
const fs = require('fs');
const pPath = path.join(path.resolve(__dirname), '../package.json');

global.sdk_version = "1.1.3";

// 定义版本和参数选项
program
    .version(JSON.parse(fs.readFileSync(pPath)).version, '-v, --version')
    .option('--init', 'initialize a project')
    .option('--pull', 'pull source code from server')
    .option('--push', 'push local source code to server')
    .option('--update', 'update remote model jar and replace locally with the latest')
    .option('--download', 'download remote model jar and replace locally with the latest')
    .option('--add', 'add an environment config')
    .option('--remove', 'remove an environment config')
    .option('--list', 'get all environment name')
    .option('--switch', 'switch current environment config')
    .option('--current', 'get current config name')
    .option('--reLogin', 're login system');

// 必须在.parse()之前，因为node的emit()是即时的
program.on('--help', function(){
    console.log('');
    console.log('');
    console.log(
        chalk.green(
            figlet.textSync("XSY CLI")
        ));
});

program.parse(process.argv);

if(program.reLogin) {
    environment.reLogin();
}

if(program.current) {
    environment.current();
}

if(program.add) {
    environment.add(function (res) {
        process.exit(0);
    });
}

if(program.remove) {
    environment.remove();
}

if(program.list) {
    environment.list();
}

if(program.switch) {
    environment.change();
}



if(program.pull) {
    environment.isLogin(function (flg) {
        if (!flg) {
            console.log("Please Login!".blue);
            loginUtils.login();
        } else {
            pull.main();
        }
    })
}

if(program.push) {
    environment.isLogin(function (flg) {
        if (!flg) {
            console.log("Please Login!".blue);
            loginUtils.login();
        } else {
            push.main();
        }
    })
}

if(program.update) {
    environment.isLogin(function (flg) {
        if (!flg) {
            console.log("Please Login!".blue);
            loginUtils.login();
        } else {
            update.main();
        }
    })
}

if(program.download) {
    environment.isLogin(function (flg) {
        if (!flg) {
            console.log("Please Login!".blue);
            loginUtils.login();
        } else {
            download.main();
        }
    })
}

if (program.init) {
    environment.isLogin(function (flg) {
        if (!flg) {
            console.log("Please Login!".blue);
            loginUtils.login();
        } else {
            init.main();
        }
    })
}


